<?php
/*
 * webtrees: online genealogy
 * Copyright (C) 2016 webtrees development team
 * Copyright (C) 2016 JustCarmen
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
namespace Andre\WebtreesAddOn\MyExplorer\Template;

/*
*	V-1-1 for My explorer V-1-7-9-2b 10 JAN 2018 ADMIN
* 	site http://www.adumas.1s.fr/wt     http://adumas.shost.ca/    http://webtrees.adumas.shost.ca/
*
*/

use Fisharebest\Webtrees\Auth;
use Fisharebest\Webtrees\Controller\PageController;
use Fisharebest\Webtrees\Filter;

use Fisharebest\Webtrees\I18N;
use Fisharebest\Webtrees\Site;
use Fisharebest\Webtrees\File;
use Fisharebest\Webtrees\Individual;
use Fisharebest\Webtrees\Tree;
use Fisharebest\Webtrees\Module;


use Andre\WebtreesAddOn\MyExplorer\myExplorerModule;

use Andre\WebtreesAddOn\MyExplorer\myUpdateFunctions;


class AdminTemplate extends myExplorerModule {

	const CUSTOM_VERSION = '1-1';
	const CUSTOM_VERSION_DATE = '23-11-2017';

	protected function pageContent() {
		$controller = new PageController;
		return
			$this->pageHeader($controller) .
			$this->pageBody($controller);
	}

	private function pageHeader(PageController $controller) {
		$js = WT_MODULES_DIR . $this->getName() . '/js/explorer.js';
		
	$msgFieldEmpty = I18N::translate('Fields with a * can not be empty !! ');
		
		$controller
			->restrictAccess(Auth::isAdmin())
			->setPageTitle($this->getTitle())
			->pageHeader()
			->addInlineJavascript('var modName =\'' . $this->getName() .'\';')
			->addExternalJavascript( $js)
			->addExternalJavascript(WT_AUTOCOMPLETE_JS_URL)
			->addInlineJavascript('autocomplete();')			
			
			->addInlineJavascript('
							function include_css(css_file) {
								var html_doc = document.getElementsByTagName("head")[0];
								var css = document.createElement("link");
								css.setAttribute("rel", "stylesheet");
								css.setAttribute("type", "text/css");
								css.setAttribute("href", css_file);
								html_doc.appendChild(css);
							}
							include_css("' . WT_MODULES_DIR . $this->getName() . '/' . 'css/' . 'myexplorer.css'.'");
							
						')						
			;						
	}
	
	/*
	////////////  Body //////////////////////
	*/
	private function pageBody(PageController $controller) {
		global $WT_TREE;
		
		static $ariane ;
		
		
		$hidden_ext = array('php'=>true, 'exe'=>true, 'tmp'=>true, 'bak'=>true, 'zip'=>true, '7z'=>true);
		if (null === $this->getSetting('MY_EXPLORER_HIDDEN_EXT')) {
			$this->setSetting('MY_EXPLORER_HIDDEN_EXT', serialize($hidden_ext));
		} else {
			$hidden_ext = unserialize($this->getSetting('MY_EXPLORER_HIDDEN_EXT'));
		}
		
		$files = array();
		/////////////////////
		// Admin page content
		if (!is_dir(WT_DATA_DIR . 'myexplorer_cache')){
			File::mkdir(WT_DATA_DIR . 'myexplorer_cache');
		}
		
		$fpdf = $this->filename; 
	
		$html = '';
		
		/////////////////////

		$ariane[0] = 'Root > ';
		$update = false;
		
		
	if (!($refdir = Filter::post('refdir'))){
		if ( !( $refdir = $this->getSetting('MY_EXPLORER_BASE_DIR') ) ){	
			$refdir = trim($this->get_WT_RootDir() , DIRECTORY_SEPARATOR);		
		}
	} else {
			$this->setSetting('MY_EXPLORER_BASE_DIR', trim(Filter::post('refdir'), DIRECTORY_SEPARATOR));
			if (file_exists(WT_DATA_DIR . $this->getSetting('MY_EXPLORER_CACHE_DIR') . DIRECTORY_SEPARATOR . $this->getSetting('MY_EXPLORER_CACHE_FILE')) ){
				unlink(WT_DATA_DIR . $this->getSetting('MY_EXPLORER_CACHE_DIR') . DIRECTORY_SEPARATOR . $this->getSetting('MY_EXPLORER_CACHE_FILE'));
			}	
	}

	$refdir = str_replace('\\', '/', $refdir);

	if (Filter::get('dirup') ){
			$refdir = dirname($refdir);		
	}
		
		// vérifier dernier /	
		
	if ( ($update = Filter::postbool('reload')) ){
			$cur = Filter::post('cur');
			$refdir = Filter::post('refdir');
		}  elseif (null === $this->getSetting('LAST_MY_EXPLORER_OPENED_DIR')) {
			$cur = $refdir;
		} else {
			$cur = $this->getSetting('LAST_MY_EXPLORER_OPENED_DIR');
		}	
	
	
		$sdata1 = null;
		$sdata2 = null;
		$sext = null;
		$scase = false;
		
				
		if (Filter::post('search')){
			if (($sdata1 = Filter::post('sdata1')) == '%?%'){$sdata1 = null;};
			if (($sdata2 = Filter::post('sdata2')) == '?'){$sdata2 = null;};
			$scase = Filter::post('scase');
			if ((($sext = Filter::post('sext')) == '' ) || (strlen($sext) > 3) ){$sext = null;};
			$this->setSetting('MY_EXPLORER_LAST_SEARCH', serialize(array('sdata1'=>$sdata1, 'sdata2' => $sdata2, 'ext' => $sext, 'scase' => $scase) ));
		}

		$js = "var modName ='" . $this->getName() ."';
				var refdir ='" . $refdir ."';
				var opendir = '" . $cur . "';
			";
		

		$action = 'module.php?mod=' . $this->getName() . '&mod_action=admin_config';
		
?>
<script><?php echo $js ?></script>
<?php echo $this->getBandeau($controller); ?>
		<fieldset><legend><?php echo I18N::translate('Root directory') ?>:</legend>
			<form name="refdir"  method="post" action="<?php echo $action ?>">
				<input type="text" 
					title="<?php echo I18N::translate('Root dir is \'.\', enter relative path like \'./modules_v3\' for example') ?>" 
					name="refdir" id="refdir" value="<?php echo $refdir ?>" style="width:60%;padding:5px;" />
				<input type="submit" value="<?php echo I18N::translate('Save') ?>"/>
			</form>
			</fieldset>
				<p></p>
	<div id="root" style="width:960px;">
			<span class="sfolder"><?php echo I18N::translate('Root') ?>  > <b>'webtrees -> .'</b> </span>
			<p id="hiddenext"><span><?php echo I18N::translate('Hidden files') ?>: &nbsp; </span>
			<?php 
				foreach ($hidden_ext as $key => $ext){ ?>
					<label><?php echo $key . ': '; ?></label><input type="checkbox" id="<?php echo $key ?>" name="<?php echo $key ?>" <?php echo   (($ext === true) ? 'checked="checked' : '')  ?>" />&nbsp;
			<?php
				}
			?>
			</p>
			<form method="post" action="<?php echo $action  ?>">
			<span>
				<input type="text" 
					title="<?php echo I18N::translate('co% for files starting with co, %oj for files name ending with ol') ?>" 
					id="sdata1" name="sdata1" value="%?%">
				
				<input type="text" id="sdata2" name="sdata2" value="">
				<input type="text" id="sext" name="sext" value="<?php echo I18N::translate('Searched extension') ?>">
				<span><input type="submit"  id="search" name="search" value="<?php echo I18N::translate('Select') ?>"/></span>
			</span>
			</form>
			<p></p>
		<table class="explorateur"><tr>
		<td valign="top" class="tdleft stitle">
			<h2 title="<?php  
				echo I18N::translate('Click here to close media window')
			?>"><?php echo	I18N::translate('Explorer') . ':' . $sdata1 . ' : ' . $sdata2 . ' : ' . $sext  ?></h2>			
				<?php 
					//$refdir = $this->getSetting('MY_EXPLORER_BASE_DIR');
					$refdir = trim($refdir, '/');					
					echo  '<div id="fdirs">';
			
					if (false === ($sdirs = $this->cacheFileDirsExists())){
						echo '<li id="liroot" class="dir-open" rel="' .$refdir . '">' . 'Root' . '</li><br>';
						$this->listOfDirs($refdir,rawurldecode($refdir),1);
					} else {
						echo $sdirs;
					}	
					?>
			</div>		
		</td>
		<td width="10"></td>
		<td valign="top" class="tdright txttitle"><h2 title="<?php  
				echo I18N::translate('Click here to close media window')
			?>">
			<?php echo substr($cur, strlen($refdir) + 1) ?>
		</h2>		
			<div class="smedia">
				<?php $file = ''; $this->printImage($file); ?> 				
			</div>
			<div id="ffiles">
				<?php 
					echo $this->listOfFiles($cur); ?>
			</div>
		</td>
		</tr></table>
		
		
<?php		
	//return true;	
	
	}
	

/* liste des fichiers 
*	selectInclude('sname' => '%?%', 'sext' => 'ext')
*	selectExclude('sname' => '%?%', 'sext' => 'ext')
*/
protected function listOfFiles($cur) {
  global $asc, $order;
	$PHP_SELF = 'module.php?mod=' . $this->getName() . '&mod_action=admin_config' ;
	if (null ===  $this->getSetting('MY_EXPLORER_TAB_FILES')){
		$this->setSetting('MY_EXPLORER_TAB_FILES', self::MY_EXPLORER_TAB_FILES);
	} 
	
	$cache =  WT_DATA_DIR . $this->getSetting('MY_EXPLORER_CACHE_DIR') . DIRECTORY_SEPARATOR . $this->getSetting('MY_EXPLORER_TAB_FILES');
	
	$tab = array();
	$new_sort = false;
	
	$asc = Filter::get('asc');
	if ($asc === null){
		$asc = true;
		$new_sort = false;		
	} else if ($asc == 1){
		//reload to sort files
		$asc= true;
		$order = Filter::get('order');
		$new_sort = true; //reload same page
	} else {
		//reload to sort files
		$asc= false;
		$order = Filter::get('order');
		$new_sort = true; //reload same page
	}
	
	
 
 
  
  $search = null;
  $key_ok = false;
  
  
  if ($cur != '.'){
	$current_dir = substr($cur, strlen($this->getSetting('MY_EXPLORER_BASE_DIR') + 1)  ); // from base dir
  } elseif (null === $this->getSetting('LAST_MY_EXPLORER_OPENED_DIR')) {
	  $current_dir = '.';
	  $this->setSetting('LAST_MY_EXPLORER_OPENED_DIR', '.');
  } else {
	  $current_dir = $this->getSetting('LAST_MY_EXPLORER_OPENED_DIR');
  }
  
 
	if (file_exists($cache)) { //tabs are in cache
		$tab = unserialize(file_get_contents($cache));
		$key_ok = in_array($current_dir, $tab);
	}	

	if ( $this->getSetting('MY_EXPLORER_LAST_SEARCH')  ){
	    $search = unserialize( $this->getSetting('MY_EXPLORER_LAST_SEARCH') );
	}

	if ((!$new_sort) && (!$key_ok ) ){
		if ($dir = opendir($cur)) {
		//if ($cur !=  $this->getSetting('LAST_MY_EXPLORER_OPENED_DIR')){
			/* tableaux */
		$tab_dir = array();
		$tab_file = array();
		
		/* extraction */
			while($file = readdir($dir)) {
				if (is_dir(($cur."/".$file))) {
					if(!in_array($file, array(".",".."))) {
						$tab_dir[] = $this->addScheme($file, $cur, 'dir');
					}
				} else if ( is_link($cur."/".$file) ){
					$tab_file[] = $this->addScheme($file, $cur, 'link');
				} else {
					$tab_file[] = $this->addScheme($file, $cur, 'file');
					/* Selection des fichiers */
				}
			}
			
			//cache tabs	
			$cfiles = array($current_dir => array(
							'subdirs' => $tab_dir,
							'files' => $tab_file)
							);
			$handle = fopen($cache, 'w+');
			fwrite($handle, serialize($cfiles) );
			fclose($handle);					
		}
		closedir($dir);
	} elseif ($key_ok){
		$tab_dir = $tab[$current_dir]['subdirs'];
		$tab_file = $tab[$current_dir]['files'];
		
	}  else {
		$tab_dir = array();
		$tab_file = array();
	}	
		
		
		
			// tri 
			switch ($order){
			case 'name': 
				usort($tab_dir, array($this, "cmp_name") );
				usort($tab_file,array($this, "cmp_name"));
			break;
			case 'size':
				usort($tab_dir,array($this, "cmp_size"));
				usort($tab_file,array($this, "cmp_size"));
			break;
			}
		
		
		//tri des affichages
		if ($this->getSetting('MY_EXPLORER_LAST_SEARCH')){
			$search = unserialize($this->getSetting('MY_EXPLORER_LAST_SEARCH'));
		} else  {
			$search = null;
		}

		if ($this->getSetting('MY_EXPLORER_HIDDEN_EXT')){
			$hidden_ext = unserialize($this->getSetting('MY_EXPLORER_HIDDEN_EXT'));
		} else {
			$hidden_ext = array();
		}
  
    $html = '';
	
    /* affichage */
    $html .= '<table cellspacing="0" cellpadding="0" border="0" id="ftable">';
    $html .= '<tr style="font-size:8pt;font-family:arial;min-height:25px;">
	<th style="width:10px;"><input type="checkbox" class="allfiles"></th>
    <th class="thname" dir="' .  $cur . '" >' . (($order == 'name') ? ((!$asc) ? "↑ ":"↓ "):"") .'<a href="' .$PHP_SELF . '&dir=' . rawurlencode($cur) .  '&order=name&asc=' . (($asc) ? 0 : 1) .'&order0=' .$order .'">Nom</a></th>
	<th class="thext" dir="' .  $cur . '">' . (($order == 'ext') ? ((!$asc) ? "↑ ":"↓ "):"") .'<a href="' .$PHP_SELF . '&dir=' . rawurlencode($cur) .  '&order=ext&asc=' . (($asc) ? 0 : 1) .'&order0=' .$order .'">Ext</a></th>
    <th class="thsize" dir="' .  $cur . '">' . (($order == 'size') ? ((!$asc) ? '↑ ' : '↓ ' ) : '') . '<a href="' .$PHP_SELF . '&dir=' . rawurlencode($cur) .  '&order=size&asc=' . (($asc) ? 0 : 1)  . '&order0=' . $order. '">Taille</a></th>
    <th>' . (($order== 'date') ? ((!$asc) ? '↑ ' : '↓') : '').'<a href="' .$PHP_SELF . '&dir=' . rawurlencode($cur) . '&order=date&asc=' . (($asc) ? 0 : 1)  . '&order0=' . $order .'">Dernière <br>  modification</a></th>
    <th>' . (($order=='type') ? ((!$asc) ? '↑ ' : '↓ ') : '') . '<a href="' .$PHP_SELF . '&dir=' . rawurlencode($cur) . '&order=type&asc=' .(($asc) ? 0 : 1)  . '&order0=' . $order . '">Type</a></th>
    <th>' . (($order == 'ext') ? ((!$asc) ? '↑ ' : '↓ ') : '') . '<a href="' .$PHP_SELF . '&dir=' . rawurlencode($cur) . '&order=ext&asc=' . (($asc) ? 0 : 1)  . '&order0=' . $order . '">Type</a></th>
    <th>' . (($order== 'perms') ? ((!$asc) ? '↑ ' : '↓ ') : '') . '<a href="' .$PHP_SELF . '&dir=' . rawurlencode($cur) . '&order=perms&asc=' . (($asc) ? 0 : 1)  . '&order0='. $order . '">Permissions</a></th>
    <th>' . (($order == 'access') ? ((!$asc) ? '↑ ' : '↓ ') : '') . '<a href="' .$PHP_SELF . '&dir=' . rawurlencode($cur) . '&order=access&asc=' . (($asc) ? 0 : 1)  . '&order0=' . $order . '">Dernier <br> accès</a></th></tr>';

	$cline = 0;
    foreach($tab_dir as $elem) {		
      $html .= '<tr>
	  <td style="width:10px;"></td>
	  <td  style="background-color: ' . $this->getLineColor($cline) . ';text-align:left;"><li class="dir-close"  rel="' . $cur . '/' .$elem['name'] . '">&nbsp; ' .  $elem['name'] . '</li></td>
      <td  style="width:80px;background-color: ' . $this->getLineColor($cline) . ';">&nbsp;</td>
	  <td  style="background-color: ' . $this->getLineColor($cline) . ';">&nbsp;</td>
      <td  style="background-color: ' . $this->getLineColor($cline) . ';">' . date("d/m/Y H:i:s" ,  $elem['date']) . '</td>
      <td  style="background-color: ' . $this->getLineColor($cline) . ';">' . $this->assocType($elem['type']) . '</td>
      <td  style="background-color: ' . $this->getLineColor($cline) . ';">&nbsp;</td>
      <td  style="background-color: ' . $this->getLineColor($cline) . ';">' . $elem['perms'] . '</td>
      <td  style="background-color: ' . $this->getLineColor($cline) . ';">' . date("d/m/Y", $elem['access']) . '</td></tr>';
    $cline++;
	}
	$cline = 0;
    foreach($tab_file as $elem) {
		if ( !array_key_exists(strtolower($elem['ext']), $hidden_ext) || (!$hidden_ext[strtolower($elem['ext'])]) ){
			
			if (  ($search['ext'] != null ) && (strtolower($search['ext']) != strtolower($elem['ext']) ) ){$bext = false; continue;}
			$bext = true;
			if ( ($search['sdata2'] != null ) && (strpos( $elem['name'], $search['sdata2']) === false ) ){$bdata2 = false; continue;}
			$bdata2 = true;
			if ( ($search['sdata1'] != null ) && ( strpos( $elem['name'], trim($search['sdata1'], '%')) === false )  ){
				$bdata1 = false;
				continue;
			} elseif ($search['sdata1'] != null ) {
				$bdata1 = true;
				$p = strpos( $elem['name'], trim($search['sdata1'], '%'));
					if ( (substr($search['sdata1'], -1)  == '%')  && $p > 0  ){continue;}
					
				if ( ($search['sdata1'][0] == '%') && $p != ( strlen($elem['name']) -4 - (strlen(trim($search['sdata1'], '%')) ) )  ){continue;}	
					
			}
			switch ($elem['ext']){
			case 'pdf':
			case 'PDF':
				$pdf = WT_MODULES_DIR . $this->getName() .  '/css/images/pdf.png';
			break;
			default:
				$pdf = '';
			break;	
			}
			
			$html .= "<tr style=\"background-color: " . $this->getLineColor($cline) . ";\">
				<td class=\"ln" . $cline . "\" style=\"width=\"10px\" background-color: " . $this->getLineColor($cline) . ";\"><input class=\"ln" . $cline . "\" type=\"checkbox\"></td>
				<td class=\"ln" . $cline . "\" style=\"background-color: " . $this->getLineColor($cline) . ";\" style=\"text-align:left;\"><li class=\"filen ln" . $cline . "\" rel=\""  . $cur . "/" .$elem['name'] . "\" >&nbsp;".$elem['name']."</li></td>
				<td class=\"ln" . $cline . "\" style=\"background-color: " . $this->getLineColor($cline) . ";\">&nbsp;" .  $elem['ext'] . "</td>
				<td class=\"ln" . $cline . "\" style=\"background-color: " . $this->getLineColor($cline) . ";\" align=\"right\">" . $this->formatSize($elem['size']) . "</td>
				<td class=\"ln" . $cline . "\" style=\"background-color: " . $this->getLineColor($cline) . ";\">" . date("d/m/Y H:i:s", $elem['date']) . "</td>
				<td class=\"ln" . $cline . "\" style=\"background-color: " . $this->getLineColor($cline) . ";\">" . $this->assocType($elem['type']) . "</td>
				<td  class=\"ln" . $cline . "\" style=\"background-color: " . $this->getLineColor($cline) . ";\">".$this->assocExt($elem['ext'])."</td>
				<td class=\"ln" . $cline . "\" style=\"background-color: " . $this->getLineColor($cline) . ";\">".$elem['perms']."</td>
				<td class=\"ln" . $cline . "\" style=\"background-color: " . $this->getLineColor($cline) . ";\">" . 
						date("d/m/Y", $elem['access']) . 
						"</td>
					</tr>";
				$cline++;
				
			}	
		}
		
	$html .= "<tr style=\"background-color: " . $this->getLineColor($cline++) . ";\">
				<td style=\"width:10px;\"></td>
				<td style=\"background-color: " . $this->getLineColor($cline) . ";\" style=\"text-align:left;\" colspan=\"8\">
				
					<input type=\"button\" class=\"delFile\" value=\"" . I18N::translate('Delete selected files...') . "\"/>
				</td>";

    $html .= "</tr></table>";
  
  return $html;
} 


/* color in table*/
private function getLineColor($line){
	if (($line %2) == 0){
			$bgd = 'white';
		} else {
			$bgd = 'lightgrey';			
		}
	return $bgd;
}

/* infos à extraire */
private function addScheme($entry,$base,$type) {
	if ($type == 'link'){
		$tab['name'] = $entry;
		$tab['type'] = 'link';
		$tab['date'] = '';
		$tab['size'] = '';
		$tab['perms'] = '';
		$tab['access'] = '';
		$tab['ext'] = '';

	} else {
		  $tab['name'] = $entry;
		  $tab['type'] = filetype($base."/".$entry);
		  $tab['date'] = filemtime($base."/".$entry);
		  $tab['size'] = filesize($base."/".$entry);
		  $tab['perms'] = fileperms($base."/".$entry);
		  $tab['access'] = fileatime($base."/".$entry);
		  $t = explode(".", $entry);
		  $tab['ext'] = $t[count($t)-1];
		}
  return $tab;
}

private function cmp_name($a = array(),$b = array()) {
	global $asc;

    if ($a['name'] == $b['name']) return 0;
    if($asc) {
        return ($a['name'] < $b['name']) ? -1 : 1;
    } else {
        return ($a['name'] > $b['name']) ? -1 : 1;
    }
}

private function cmp_size($a = '',$b = '') {
	global $asc;
 
    if ($a['size'] == $b['size']) return 0; //cmp_name($a,$b);
    if($asc) {
        return ($a['size'] < $b['size']) ? -1 : 1;
    } else {
        return ($a['size'] > $b['size']) ? -1 : 1;
    }
}

/* formatage de la taille */
private function formatSize($s) {
  /* unités */
  $u = array('o','Ko','Mo','Go','To');
  /* compteur de passages dans la boucle */
  $i = 0;
  /* nombre à afficher */
  $m = 0;
  /* division par 1024 */
  while($s >= 1) {
    $m = $s;
    $s /= 1024;
    $i++;
  }
  if(!$i) $i=1;
  $d = explode(".",$m);
  /* s'il y a des décimales */
  if($d[0] != $m) {
    $m = number_format($m, 2, ",", " ");
  }
  return $m." ".$u[$i-1];
}
/*
La fonction assocType() permet de donner une description francophone du type d'entité via un tableau de correspondance.
 
Sélectionnez
*/
/* formatage du type */
private function assocType($type) {
  /* tableau de conversion */
  $t = array(
    'fifo' => "file",
    'char' => "fichier spécial en mode caractère",
    'dir' => "dossier",
    'block' => "fichier spécial en mode bloc",
    'link' => "lien symbolique",
    'file' => "fichier",
    'unknown' => "inconnu"
  );
  return $t[$type];
}


/* description de l'extention */
private function assocExt($ext) {
  $e = array(
    '' => "inconnu",
    'doc' => "Microsoft Word",
    'xls' => "Microsoft Excel",
    'ppt' => "Microsoft Power Point",
    'pdf' => "Adobe Acrobat",
    'zip' => "Archive WinZip",
    'txt' => "Document texte",
	'TXT' => "Document texte",
    'gif' => "Image GIF",
	'GIF' => "Image GIF",
    'jpg' => "Image JPEG",
	'JPG' => "Image JPEG",
    'png' => "Image PNG",
	'PNG' => "Image PNG",
    'php' => "Script PHP",
    'php3' => "Script PHP",
    'htm' => "Page web",
    'html' => "Page web",
    'css' => "Feuille de style",
    'js' => "JavaScript"
  );
  if(in_array($ext, array_keys($e))) {
    return $e[$ext];
  } else {
    return $e[''];
  }
} 

	
	
	/*
	
	*
	*/
	private function checkboxesOptionsExt(){
		
	}
	
	/**
	 * Print the  thumbnail for this historical fact
	 *
	 * @param $file
	 * @return thumbnail
	 */
	protected function printImage($filename) {
		
		$link = '';
		$link1 = '';
		
	//		$filename = $this->myDir() . 'thumbs/' . $file;
			$file = basename($filename);
			//$mimetype = finfo::file ( $filename , FILEINFO_MIME_TYPE);	
			$mimetype = $this->getMimeType($file);
			if (is_file($filename)) {
				$imgsize = getimagesize($filename);
				$image	 = '<img style="width-max:80%;" ' .
					' dir="' . 'auto' . '"' . // For the tool-tip
					' src="module.php?mod=' . $this->getName() . '&amp;mod_action=display_img&amp;file=' . $filename . '&amp;thumb=2&amp;cb=' . trim(md5_file($filename), '"\'') . '"' .
					' alt="' . ' ' . '"' .
					' title="' . $file . '"' .
					' data-cachefilename="' . basename($filename) . '"' .
					' ' . $imgsize[3] . // height="yyy" width="xxx"
					'>';
				
					$html = '<a' .
					' class="' . '' . '"' .
					' href="' . $link . '"' .
					' type="' . $mimetype . '"' .
					' data-obje-url="' . $link1 . '"' .
					' data-obje-note="' . ' ' . '"' .
					' data-title="' . $file . '"' .
					'>' . $image . '</a>';
					
					return $html;
			} else {
				return '<a></a>';
			}
		
	}

	
	// root apache
	private function getRoot(){
  return $_SERVER["DOCUMENT_ROOT"];//."/../";
}

	// root dir webtreees
	private function get_WT_RootDir(){
	//return  substr(WT_DATA_DIR,0 , -5);
	return '.';
}	

/**
 * Search a folder (and optional subfolders) for filenames that match a search pattern.
 *
 * @param string $dir
 * @param bool   $recursive
 * @param string $filter
 *
 * @return string[]
 */
 
 //////////////////////////////////////////////////
//http://lemon-cake.fr/explorateur-de-fichiers/



private function listOfDirs($base, $cur, $level=0) {
	ini_set("allow_url_fopen", true );
	$tab = array();
  if ($dir = opendir($base)) {	 
    
		while ($entry = readdir($dir)) {
			if (is_dir($base."/".$entry) && !in_array($entry, array(".",".."))) {
				$tab[] = $entry;
			}
		}
		sort($tab);
		echo  '<ul class="tree">';
	
		foreach ($tab as $entry) {
			/* chemin relatif à la racine */
			  $file = $base."/".$entry;
			 /* l'entrée est-elle le dossier courant */
				if ($file == $cur) {
					echo  '<li class="dir-open" rel="' .$base  . '/' . $entry . '">' . $entry . '</li>';
				} else {
					echo '<li class="dir-close" rel="' . $base .'/'.$entry . '" >' .$entry . '</li>';
				}
				/* l'entrée est-elle dans la branche dont le dossier courant est la feuille */
				if (strpos($file."/",$cur."/") !== false) {
					$this->listOfDirs($file, $cur, $level+1);
				}	
		}
	echo '</ul>';
    closedir($dir);
	//$this->saveDirs($base, $tab);	
  }
}

private function delCacheFile(){
	if (file_exists(
		($cache =  WT_DATA_DIR . $this->getSetting('MY_EXPLORER_CACHE_DIR') . DIRECTORY_SEPARATOR . $this->getSetting('MY_EXPLORER_CACHE_FILE'))
		)){
		unlink($cache);
	}
	return true;
}

private function isset_cacheDir(){
	if (!is_dir( WT_DATA_DIR . $this->getSetting('MY_EXPLORER_CACHE_DIR'))){
		File::mkdir(WT_DATA_DIR . $this->getSetting('MY_EXPLORER_CACHE_DIR'));
	}
	return true;
}


private function dirFromCache($base){
	
	$tab = $this->cacheDirsExists($base); 
	if ((false === ($tab)) ){ 
		return false;
	} else {
		return $tab;
	}
	
}

/*

$tab = $this->cacheDirsExists();
	if ((false === ($tab)) ){ 
	}
*/
/*
*	cache dirs structure
*	reset when you save a new//old  root dir
*/

protected function cacheAllDirs($html){
	$base = $this->getSetting('MY_EXPLORER_BASE_DIR');
	if (!is_dir( WT_DATA_DIR . $this->getSetting('MY_EXPLORER_CACHE_DIR'))){
		File::mkdir(WT_DATA_DIR . $this->getSetting('MY_EXPLORER_CACHE_DIR'));
	}
	$handle = fopen(WT_DATA_DIR . $this->getSetting('MY_EXPLORER_CACHE_DIR') . DIRECTORY_SEPARATOR . $this->getSetting('MY_EXPLORER_CACHE_FILE'), 'w+');
	fwrite($handle, serialize(array($base => $html)));
	fclose($handle);
	return true;
}

private function saveDirs($base, $tab){
	$dirs = array($base => $tab);
	if (!is_dir( WT_DATA_DIR . $this->getSetting('MY_EXPLORER_CACHE_DIR'))){
		File::mkdir(WT_DATA_DIR . $this->getSetting('MY_EXPLORER_CACHE_DIR'));
	}
	$handle = fopen(WT_DATA_DIR . $this->getSetting('MY_EXPLORER_CACHE_DIR') . DIRECTORY_SEPARATOR . $this->getSetting('MY_EXPLORER_CACHE_FILE'), 'w+');
	fwrite($handle, serialize($dirs));
	fclose($handle);
	return true;
}

/*
*	return true/false
*/
 private function cacheFileDirsExists(){
	 $refdir = $this->getSetting('MY_EXPLORER_BASE_DIR');
	if (!($this->isset_cacheDir() )){
		return false;
	} 
	if (!file_exists(WT_DATA_DIR . $this->getSetting('MY_EXPLORER_CACHE_DIR') . DIRECTORY_SEPARATOR . $this->getSetting('MY_EXPLORER_CACHE_FILE'))){
		return false;
	} 
	if (!array_key_exists($refdir,($tmp = unserialize(
														file_get_contents(
		WT_DATA_DIR . $this->getSetting('MY_EXPLORER_CACHE_DIR') . DIRECTORY_SEPARATOR . $this->getSetting('MY_EXPLORER_CACHE_FILE')												
														) 
													 ) 
								  ) 
						)
		){
		return false;
	} 
/*	if (filemtime(WT_DATA_DIR . $this->getSetting('MY_EXPLORER_CACHE_DIR') . DIRECTORY_SEPARATOR . $this->getSetting('MY_EXPLORER_CACHE_FILE')) > WT_TIMESTAMP - 86400 ){
		return false;
	} */
	//$this->writeDirs($tmp[$refdir]);
/*	return '<script>
				$("div#fdirs").empty().append(' . $tmp[$refdir]  . ');
			</script>';
	*/		
	return 	$tmp[$refdir];	
 }

 
 private function writeDirs($tab){
		echo  '<ul class="tree">';
	
		foreach ($tab as $entry) {
			/* chemin relatif à la racine */
			  $file = $base."/".$entry;
			 /* l'entrée est-elle le dossier courant */
				if ($file == $cur) {
					echo  '<li class="dir-open" rel="' .$base  . '/' . $entry . '">' . $entry . '</li>';
				} else {
					echo '<li class="dir-close" rel="' . $base .'/'.$entry . '" >' .$entry . '</li>';
				}
				/* l'entrée est-elle dans la branche dont le dossier courant est la feuille */
				if (strpos($file."/",$cur."/") !== false) {
		//?????			$this->listOfDirs($file, $cur, $level+1);
				}	
		}
	echo '</ul>'; 
 }

protected function scan_dir1b1($dir, $search = null, $search1 = null, $scase = false, $sext = null) { 
	set_time_limit(0);
	static $html = '';
	static $boucles = 0;
	if ($boucles  == 2){return $html;}
	//$search1 = null;
	if ($search1 !== null){$search1 = trim($search1, '%');}
	
	
    // On regarde déjà si le dossier existe
    if (is_dir($dir)) {
        // On le scan et on récupère dans un tableau le nom des fichiers et des dossiers
        $files = scandir($dir);

        // On supprime . et .. qui sont respectivement le dossier courant et le dossier précédent
        unset($files[0], $files[1]);

        // On tri le tableau de façon intéligente (à la façon humaine)
        // http://www.php.net/function.natcasesort
        natcasesort($files);
		
        // On commence par afficher les dossiers
		$html .= '<li class="folder, folder-1" >'.'..'.'</li>';
        foreach($files as $f) {
            // S'il y a un dossier
            if (is_dir($dir.$f)) {
                // On affiche alors les données
               $html .= '<li class="folder">'.$f.'</li>';
                $html .= '<ul class="tree">';

                // Et du coup comme c'est un dossier, un le rescan
				$boucles++;
				$this->scan_dir1b1($dir.$f."/", $search, $search1, $scase, $sext);
	
                $html .=  '</ul>';
            }
        }
		

		//init la recherche	
		if (null !== $search){
			$bs = false;
			$be = false;

			
			if ($search[0] == '%'){$bs = true;}
			if (substr($search, -1) == '%'){$be = true;}
			//$search = trim($search, '%');
		}
		
        // Puis on affiche les fichiers correspondants à la recherche
		$nondisplay = array('bak', 'tmp', 'php', 'js', 'exe');
		//echo $dir;
        foreach($files as $f) {
            // S'il y a un fichier
            if ( is_file($dir.$f) ) {
				//élimine les bak, tmp..
				$tmp = explode('.', $f);
				$ext =  end($tmp);
				if (!$scase){$ext = strtolower($ext);}
				if ( in_array($ext, $nondisplay) ){
					continue;
				}
				
				if (($sext !== null) && ($ext != $sext)  ){
					continue;
				}
				if ($search === null){
					$html .= '<li class="file" rel="'.$dir.$f.'">'.$f.'</li>';
				} else { 
					if ($search1 === null){$bsub = true;$subr = false;} else {$bsub = false;}
					
					//maj/min
					if (!$scase){
						$r = stripos($f, trim($search, '%'));
						if ($search1 !== null){
							$subr = stripos($f, trim($search1, '%')  );
						}	
					} else {
						$r = strpos($f, trim($search, '%'));
						if ($search1 !== null){
							$subr = strpos($f, trim($search1, '%'));
						}
					}
					
					if (false !== $subr){
						if ($subr >= 0 ){$bsub = true;} 
					}
					
					if ($r !== false){
						if (!$be  && !$bs){ //pas de %
							if (($r >= 0) && $bsub ){
								$html .= '<li class="file" rel="'.$dir.$f.'">' . $f.'</li>';
							}	
						} 
						
						if (!$be  && $bs){ //%xx
							if (($r == ( strlen($f) - strlen(trim($search, '%')))) && $bsub ){
								$html .= '<li class="file" rel="'.$dir.$f.'">' . $f.'</li>';
							}
						} 
						
						if ($be  && !$bs){//gh% 
							if (($r == 0)  && $bsub){
								$html .= '<li class="file" rel="'.$dir.$f.'">' . $f.'</li>';
							}
						} 
						
						if ($be  && $bs){// %hhjh% 
							if (($r >= 0) && $bsub) {
								$html .= '<li class="file" rel="'.$dir.$f.'">' . $f.'</li>';
							}
						}
					}	
				}
			}
        }
    }
	return $html;
}
 
 
 protected function fscan_dirs($adr = null){
	 $adr = 'http://localhost:8081/webtrees/';
	$handle = opendir($adr, 'r+');
	
	 
 }
 
	protected  function chooseBaseDir($dir = self::SOURCES_BASEDIR){
			static $ariane = array();
			$ariane[] = $dir; 
		// Ouvre un dossier bien connu, et liste tous les dossiers
		$html = '<select name="dossiers" id="dossiers" style="Background-color:lightgrey;padding:10px;">
			<option value="0">' . I18N::translate('List of all direstories') . '';
		if (is_dir($dir)) {			
				foreach (scandir($dir) as $file) {
					//if (is_dir($file) && $file != '.' && $file != '..'){
						$html .= '<option value="' . $file . '"><span class="indirs" style="font-size:0.8em;"> ► </span>' . $file . '</option>';
					//}	
				}
			
		}	
				
		$html .= '</select>';
	return $html;	
	}
	
	/*
	*
	*/
	private function getSelectedPaper(){
		$html = '<select id="ftype" name="ftype">
					<option value="0">' . I18N::translate('What type of source?') . '</option>
					<option value="N_">' . I18N::translate('Birth') . '</option> 
					<option value="D_">' . I18N::translate('Death') . '</option> 
					<option value="M_">' . I18N::translate('Marriage') . '</option> 
					<option value="CM_">' . I18N::translate('Marriage contract') . '</option> 
					<option value="TST_">' . I18N::translate('Will') . '</option> 
					<option value="AN_">' . I18N::translate('Others') . '</option>
					<option value="CENS_">' . I18N::translate('Cens') . '</option> 
				</select>	
			';
	return $html;	
	}
	
		/////////////////// Header ///////////////////

	private function getBandeau($controller){
		$linkdoc = Site::getPreference('MY_VERSIONS_DOC_BASE_URL') . $this->getName() . '.html';
		$linklog = Site::getPreference('MY_VERSIONS_DOC_BASE_URL') . $this->getName() . '_log.txt';
		
		//update version?
		myUpdateFunctions::checkNewModsVersions();
		$spref = Site::getPreference('LATEST_MY_MODS_VERSION');
		
		if ($spref !== null){
			//
			$tmp = unserialize($spref);
			if (in_array($this->getName(), $tmp)){
				$new_version = unserialize($spref)[$this->getName()]['version'];
			} else {
				$new_version = myExplorerModule::CUSTOM_VERSION;
			}
		} else {		
			$new_version = myExplorerModule::CUSTOM_VERSION;
		}

		$html = '<ol class="breadcrumb small">
			<li><a href="admin.php">'. I18N::translate('Control panel') . '</a></li>
			<li><a href="admin_modules.php">' . I18N::translate('Module administration') . '</a></li>
			<li class="active">' . $controller->getPageTitle() . '</li>
		</ol>';
		$html .= '<div style="
						border:2px solid black;
						border-radius:3px;
						padding:10px;">';
						
		$html .= '	<table style="width:95%;">
			<tr>
			<td style="width:65%; text-align:center;">
			<h3>' . $this->getTitle() . '  V-' . myExplorerModule::CUSTOM_VERSION . '   ' . '<span style="font-size:60%;">(' . 
							myExplorerModule::CUSTOM_VERSION_DATE . ')</span></h3>
			</td>
			<td style="text-align:center;"><span>
				<a href="' . $linkdoc . '">' .  I18N::translate('Documentation') . '
				</a></span></td>
			<td style="text-align:center;"><span>
			<a href="' . $linklog . '">' .  I18N::translate('Change.log') . '
			</a></span></td>
			</tr>';
			
		$html .= '<tr><td style="text-align:center;" colspan="3">';
		//new version?
		$html .= myUpdateFunctions::lookForNewVersion($this->getName(), myExplorerModule::CUSTOM_VERSION, $new_version ) ;
		$html .= '</td></tr>';
		
		$html .= '</table></div><br>';

	return $html;
	}
	




		/**
	 * Print the  image 
	 *
	 * @param $file
	 * @return image
	 */
	protected function printMyImage($file) {
		
		$link = '';
		$link1 = '';
		
			$filename =  basename($file);
			//$mimetype = finfo::file ( $filename , FILEINFO_MIME_TYPE);
			$mimeType = mime_content_type($file);

			
			//$mimetype = $this->getMimeType($file);
			if (is_file($filename)) {
				$imgsize = getimagesize($filename);
			//	$image	 = '<img' .
					' dir="' . 'auto' . '"' . // For the tool-tip
					' src="module.php?mod=' . $this->getName() . '&amp;mod_action=thumbnail&amp;file=' . $file . '&amp;thumb=2&amp;cb=' . trim(md5_file($filename), '"\'') . '"' .
					' alt="' . ' ' . '"' .
					' title="' . $file . '"' .
					' data-cachefilename="' . basename($filename) . '"' .
					' ' . $imgsize[3] . // height="yyy" width="xxx"
					'>';
				$image	 = '<img' .
					' dir="' . 'auto' . '"' . // For the tool-tip
					' src="' . WT_DATA_DIR . 'journal/thumbs/cesar.jpg' . '" ' .
					' alt="' . ' ' . '"' .
					' title="' . $file . '"' .
					' data-cachefilename="' . basename($filename) . '"' .
					' ' . $imgsize[3] . // height="yyy" width="xxx"
					'>';
					$html = '<a' .
					' class="' . '' . '"' .
					' href="' . $link . '"' .
					' type="' . $mimetype . '"' .
					' data-obje-url="' . $link1 . '"' .
					' data-obje-note="' . ' ' . '"' .
					' data-title="' . $file . '"' .
					'>' . $image . '</a>';
					
					return $html;
			} else {
				return '<a></a>';
			}
	}		
	


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////	
	private function getTableOfFiles($refdir){
		$html = '<ul>';
		$directory = new \RecursiveDirectoryIterator($refdir, \FilesystemIterator::FOLLOW_SYMLINKS);
		$iterator = new \RecursiveIteratorIterator($directory, \RecursiveIteratorIterator::SELF_FIRST);
		
		foreach ($iterator as $file){
			
			if ($file->isDir()){
				if ($file->getBasename() == '.' ){continue;}
				if ($file->getBasename() == '..'){					
					continue;
				}
				$html .= '</ul>';
				$html .= '<li class="folder">'. $file->getBasename() .'</li>';
				$html .= '<ul class="tree">';
                
				
			} else {//file
				$html .= '<li class="file" rel="'.$file->getPathname().'">' . $file->getBasename() . '</li>';
			}
			
			
		}
		return $html;
		
		/*  tri sur chaque level/levelfunction BuildTree($it, $separator = '  ', $level = '') {
    $results = array();
    foreach ($it as $file) {
        if (in_array($file->getBasename(), array('.', '..'))) {
            continue;
        }
        $tmp = $level . $file->getBaseName();
        if ($it->hasChildren()) {
            $newLevel = $level . $separator;
            $tmp .= "\n" . BuildTree($it->getChildren(), $separator, $newLevel);
        }
        $results[] = $tmp;
    }
    natsort($results);
    return implode("\n", $results);
};
$it = new RecursiveDirectoryIterator($dir);
$tree = BuildTree($it);

It's a pretty simple recursive parser, and does the natural sort on each level. 
		
		*/
	}


		private function getTableOfFiles1($refdir){
		$html = '<ul class="tree">';
		$directory = new \RecursiveDirectoryIterator($refdir, \FilesystemIterator::FOLLOW_SYMLINKS);
		//$iterator = new \RecursiveIteratorIterator($directory, \RecursiveIteratorIterator::SELF_FIRST);
		//$files = iterator_to_array($iterator);
		//natsort($files);
		$files = new \RecursiveTreeIterator($directory);
		foreach ($files as $file => $value){
			
			if (is_Dir($file)){
				if (basename($file) == '.' ){continue;}
				if (basename($file) == '..'){					
					continue;
				}
				$html .= '</ul>';
				$html .= '<li class="folder">'. basename($file) .'</li>';
				$html .= '<ul class="tree">';
                
				
			} else {//file
				$html .= '<li class="file" rel="'.$file.'">' . basename($file) . '</li>';
			}
			
			
		}
		return '<li class="folder">Root</li><ul class="tree">' . $html . '</ul>';
		
	}



private function getDirs1($dir) {
	$dirs = array();

	if (is_dir($dir)) {
		foreach (scandir($dir) as $path) {
			if (is_dir($dir . $path)) {
				$dirs[] = $path;
				// What if there are user-defined subfolders “thumbs” or “watermarks”?
				/*if ($path != '.' && $path != '..' && $path != 'thumbs' && $path != 'watermark' && $recursive) {
					foreach (scan_dirs($dir . $path . '/', $recursive, $filter) as $subpath) {
						$files[] = $path . '/' . $subpath;
					}
				}*/
			} 
		}
	}

	return $dirs;
}

	/*
	* List of allfiles/dirs level/level
	*/
	
	private function getAllFiles($dir){
		$files = sandir($dir);
		foreach ($files as $f){
			if (is_dir($f)){
				//encore un tour
				$file = scandir($f);
			} 
			
			if (file_exists($f)){
				
			}
		}
	}
	

/**
 * Fetch a list of all files on disk
 *
 * @param string $media_folder Location of root folder
 * @param string $media_path   Any subfolder
 * @param string $subfolders   Include or exclude subfolders
 * @param string $filter       Filter files whose name contains this test
 *
 * @return string[]
 */
private function all_disk_files($media_folder, $media_path, $subfolders, $filter) {
	return scan_dirs(WT_DATA_DIR . $media_folder . $media_path, $subfolders == 'include', $filter);
}



protected function scan_dirs_($dir, $search = null, $search1 = null, $scase = false, $sext = null) { 
	set_time_limit(0);
	static $html = '';
	//$search1 = null;
	if ($search1 !== null){$search1 = trim($search1, '%');}
	
	
    // On regarde déjà si le dossier existe
    if (is_dir($dir)) {
        // On le scan et on récupère dans un tableau le nom des fichiers et des dossiers
        $files = scandir($dir);

        // On supprime . et .. qui sont respectivement le dossier courant et le dossier précédent
        unset($files[0], $files[1]);

        // On tri le tableau de façon intéligente (à la façon humaine)
        // http://www.php.net/function.natcasesort
        natcasesort($files);
		
        // On commence par afficher les dossiers
		$html .= '<li class="folder, folder-1" >'.'..'.'</li>';
        foreach($files as $f) {
            // S'il y a un dossier
            if (is_dir($dir.$f)) {
                // On affiche alors les données
               $html .= '<li class="folder">'.$f.'</li>';
                $html .= '<ul class="tree">';

                // Et du coup comme c'est un dossier, un le rescan
                $this->scan_dirs($dir.$f."/", $search, $search1, $scase, $sext);

                $html .=  '</ul>';
            }
        }
		

		//init la recherche	
		if (null !== $search){
			$bs = false;
			$be = false;

			
			if ($search[0] == '%'){$bs = true;}
			if (substr($search, -1) == '%'){$be = true;} 
			//$search = trim($search, '%');
		}
		
        // Puis on affiche les fichiers correspondants à la recherche
		$nondisplay = array('bak', 'tmp', 'php', 'js', 'exe');
		//echo $dir;
        foreach($files as $f) {
            // S'il y a un fichier
            if ( is_file($dir.$f) ) {
				//élimine les bak, tmp..
				$tmp = explode('.', $f);
				$ext =  end($tmp);
				if (!$scase){$ext = strtolower($ext);}
				if ( in_array($ext, $nondisplay) ){
					continue;
				}
				
				if (($sext !== null) && ($ext != $sext)  ){
					continue;
				}
				if ($search === null){
					$html .= '<li class="file" rel="'.$dir.$f.'">'.$f.'</li>';
				} else { 
					if ($search1 === null){$bsub = true;$subr = false;} else {$bsub = false;}
					
					//maj/min
					if (!$scase){
						$r = stripos($f, trim($search, '%'));
						if ($search1 !== null){
							$subr = stripos($f, trim($search1, '%')  );
						}	
					} else {
						$r = strpos($f, trim($search, '%'));
						if ($search1 !== null){
							$subr = strpos($f, trim($search1, '%'));
						}
					}
					
					if (false !== $subr){
						if ($subr >= 0 ){$bsub = true;} 
					}
					
					if ($r !== false){
						if (!$be  && !$bs){ //pas de %
							if (($r >= 0) && $bsub ){
								$html .= '<li class="file" rel="'.$dir.$f.'">' . $f.'</li>';
							}	
						} 
						
						if (!$be  && $bs){ //%xx
							if (($r == ( strlen($f) - strlen(trim($search, '%')))) && $bsub ){
								$html .= '<li class="file" rel="'.$dir.$f.'">' . $f.'</li>';
							}
						} 
						
						if ($be  && !$bs){//gh% 
							if (($r == 0)  && $bsub){
								$html .= '<li class="file" rel="'.$dir.$f.'">' . $f.'</li>';
							}
						} 
						
						if ($be  && $bs){// %hhjh% 
							if (($r >= 0) && $bsub) {
								$html .= '<li class="file" rel="'.$dir.$f.'">' . $f.'</li>';
							}
						}
					}	
				}
			}
        }
    }
	return $html;
}


/**
 * Search a folder (and optional subfolders) for filenames that match a search pattern.
 *
 * @param string $dir
 * @param bool   $recursive
 * @param string $filter
 *
 * @return string[]
 */
function scan_dirs($dir, $recursive, $filter) {
	$files = array();

	// $dir comes from the database. The actual folder may not exist.
	if (is_dir($dir)) {
		foreach (scandir($dir) as $path) {
			if (is_dir($dir . $path)) {
				// What if there are user-defined subfolders “thumbs” or “watermarks”?
				if ($path != '.' && $path != '..' && $path != 'thumbs' && $path != 'watermark' && $recursive) {
					foreach (scan_dirs($dir . $path . '/', $recursive, $filter) as $subpath) {
						$files[] = $path . '/' . $subpath;
					}
				}
			} elseif (!$filter || stripos($path, $filter) !== false) {
				$files[] = $path;
			}
		}
	}

	return $files;
}

/**
 * Fetch a list of all files on disk
 *
 * @param string $media_folder Location of root folder
 * @param string $media_path   Any subfolder
 * @param string $subfolders   Include or exclude subfolders
 * @param string $filter       Filter files whose name contains this test
 *
 * @return string[]
 */
function all_disk_files_($media_folder, $media_path, $subfolders, $filter) {
	return scan_dirs(WT_DATA_DIR . $media_folder . $media_path, $subfolders == 'include', $filter);
}



	
}	