<?php
// Classes and libraries for module system
//
// webtrees: Web based Family History software
// Copyright (C) 2012 webtrees development team.
//
// Derived from PhpGedView
// Copyright (C) 2010 John Finlay
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//


/*--------------------------------------------------------------------
* http://www.adumas.1s.fr/wt/   
* André DUMAS FRANCE   DEC 2015 
* 
*	V-1-7-9-1  for webtrees 1.7.9
*		André 15-11-2017 AD  FR
*/
//============================================
 
namespace Andre\WebtreesAddOn\MyExplorer;

use Composer\Autoload\ClassLoader;
use Fisharebest\Webtrees\Controller\BaseController;
use Fisharebest\Webtrees\Controller\PageController;
use Fisharebest\Webtrees\Filter;
use Fisharebest\Webtrees\I18N;
use Fisharebest\Webtrees\Log;
use Fisharebest\Webtrees\Module\AbstractModule;
use Fisharebest\Webtrees\Module\ModuleConfigInterface;
use Fisharebest\Webtrees\Individual;
use Fisharebest\Webtrees\Family;
use Fisharebest\Webtrees\File;
use Fisharebest\Webtrees\Auth;
use Fisharebest\Webtrees\Media;
use Fisharebest\Webtrees\FlashMessages;
use Fisharebest\Webtrees\Tree;
use Fisharebest\Webtrees\GedcomRecord;
use Fisharebest\Webtrees\Database;
use Fisharebest\Webtrees\Functions;
use Fisharebest\Webtrees\Functions\FunctionsPrint;
use Fisharebest\Webtrees\Functions\FunctionsMedia;
use Fisharebest\Webtrees\Functions\FunctionsEdit;

use Andre\WebtreesAddOn\MyExplorer\myUpdateFunctions;

use PDO;

use Andre\WebtreesAddOn\MyExplorer\Template\AdminTemplate;

class myExplorerModule extends AbstractModule implements  ModuleConfigInterface {
	
	var $filename;
	
	const CUSTOM_VERSION = '1-7-9-2b';
	const CUSTOM_VERSION_DATE = '15-01-2018';
		var $dir;
		
		// How to update the database schema for this module
	const SCHEMA_TARGET_VERSION			= 1;
	const SCHEMA_SETTING_NAME			= 'MY_EXPLORER_SCHEMA_VERSION';
	const SCHEMA_MIGRATION_PREFIX		= '\Andre\WebtreesAddOn\MyExplorer\Schema';
	
	const MY_EXPLORER_CACHE_DIR 	= 'myexplorer_cache';
	const MY_EXPLORER_CACHE_FILE 	= 'cache.txt';
	const MY_EXPLORER_TAB_FILES		= 'files.txt';
	const MY_EXPLORER_BASE_DIR		= './';
	
	public function __construct() {
	global $WT_TREE;
		parent::__construct(__DIR__);
						
		$loader = new ClassLoader();
		$loader->addPsr4('Andre\\WebtreesAddOn\\MyExplorer\\', WT_MODULES_DIR . $this->getName() . '/app');
		$loader->register();
		

		
		$this->filename = WT_DATA_DIR  . 'myexplorer_cache/cache.txt';	

		/* update schema */
		Database::updateSchema(self::SCHEMA_MIGRATION_PREFIX, self::SCHEMA_SETTING_NAME, self::SCHEMA_TARGET_VERSION);

		
}


	// Extend class WT_Module
	public function getTitle() {
	return  /* I18N: Name of a module*/  I18N::translate('My explorer');
}

	// Extend class WT_Module
	public function getDescription() {
		return /* I18N: Description of the Orphan module */ I18N::translate('Displays all the dirs and files selected');
	}
// Extend class WT_Module
	public function defaultAccessLevel() {
		return Auth::PRIV_PRIVATE;
	}

	
	private function template(){
		return new AdminTemplate;
	}
	
	// Implement WT_Module_Config
	public function getConfigLink() {
		return 'module.php?mod='.$this->getName().'&amp;mod_action=admin_config';
	}

	// Implement class WT_Module_Block
	private function config(){
		$t = new AdminTemplate();
		echo $t->pageContent();
	//return true;	
	}
	
public function modAction($mod_action){
		switch ($mod_action) {
				
			case 'admin_config':					
				if (!$this->getSetting('MY_EXPLORER_CACHE_DIR')){
					Log::addConfigurationLog('myExplorer: getSetting');
					$this->setSetting('MY_EXPLORER_CACHE_DIR', self::MY_EXPLORER_CACHE_DIR);
					$this->setSetting('MY_EXPLORER_CACHE_FILE', self::MY_EXPLORER_CACHE_FILE);
					$this->setSetting('MY_EXPLORER_TAB_FILES', self::MY_EXPLORER_TAB_FILES);
					$this->setSetting('MY_EXPLORER_BASE_DIR', self::MY_EXPLORER_BASE_DIR);
				}
				if (!$this->getSetting('CUSTOM_VERSION')  || ( $this->getSetting('CUSTOM_VERSION') != self::CUSTOM_VERSION )  ){
					$this->setSetting('CUSTOM_VERSION', self::CUSTOM_VERSION);
				}
							$this->config();
			break;	
			case 'create_pdf':
					
					$this->template()->pageBody($this->filename);
			break;
			case 'change_dir':
				$dir = Filter::get('dir');
				echo $this->template()->chooseBaseDir($dir);
			break;
			case 'updateFiles';
				$t = new AdminTemplate();
		echo 	$t->pageContent();
			
			break;
			case 'explorer1b1':
				$pathdir 	= Filter::post('pathdir');
				$dir = Filter::post('dir');
				if (($sdata1 = Filter::post('sdata1')) == '%?%'){$sdata1 = null;};
				if (($sdata2 = Filter::post('sdata2')) == '?'){$sdata2 = null;};
				$scase = Filter::post('scase');
				if (($sext = Filter::post('sext')) == ''  ){$sext = null;};
				echo $this->template()->scan_dir1b1($dir, $sdata1, $sdata2, $scase , $sext);
			break;
			case 'explorer':
				$dir = $this->getSetting('MY_EXPLORER_BASE_DIR');
				if (($sdata1 = Filter::post('sdata1')) == '%?%'){$sdata1 = null;};
				if (($sdata2 = Filter::post('sdata2')) == '?'){$sdata2 = null;};
				$scase = Filter::post('scase');
				if (($sext = Filter::post('sext')) == ''  ){$sext = null;};
				$this->setSetting('MY_EXPLORER_LAST_SEARCH', serialize(array('s1' => $sdata1, 's2' => $sdata2, 'ext' => $sext, 'case'=>$scase )) ); 
				echo $this->template()->listOfFiles($dir);
			break;
			
			case 'listOfFiles':
				$cur = trim(Filter::post('cur'), DIRECTORY_SEPARATOR);
				$this->setSetting('LAST_MY_EXPLORER_OPENED_DIR', trim($cur, DIRECTORY_SEPARATOR));
				//echo $this->template()->listOfFiles($cur);
				return true;
			break;
			case 'display_file':
				$pathfile 	= Filter::post('file');
				$sext 		= Filter::post('sext');
				$file 		= basename($pathfile);
				//mimetype
				$path_parts = pathinfo($pathfile);
				$sext = $path_parts['extension'];				
				if (file_exists($pathfile) /*&& is_readable($file)*/ ){
					//readfile
					echo '<xmp>' . utf8_encode(file_get_contents($pathfile)) . '</xmp>';
				} else {
					echo 'Erreur lecture';
				}
			
			break;
			case 'cacheDirs':
				$html = Filter::post('fdirs');
				echo $this->template()->cacheAllDirs($html);
				
			
			break;
			case 'display_img':
				$pathfile 	= Filter::get('file');		
				$file 		= basename($pathfile);		
				$sext 		= self::getExtFile($file);
		//	if (file_exists($pathfile) /*&& is_readable($file)*/ ){	
				//mimetype
				$mimeType 	= self::getMimeType($file);
				$filesize		= filesize($pathfile);
				
				// send headers for the image
				header('Content-Type: ' . $mimeType);
				header('Content-Disposition: filename="' . $file . '"');
				header('Content-Length: ' . $filesize);

				if (function_exists('readfile')) {
					readfile($pathfile);
				} else {
					$fp = fopen($pathfile, 'rb');
					if (function_exists('fpassthru')) {
						fpassthru($fp);
					} else {
						while (!feof($fp)) {
							echo fread($fp, 65536);
						}
					}
					fclose($fp);
				}
				
/*			} else {
					echo 'Erreur lecture';
			}
*/			
			break;
			case 'updateHiddenExt':
				$cext = Filter::post('cext');
				$cextval = Filter::postbool('cval');
				$thext = unserialize($this->getSetting('MY_EXPLORER_HIDDEN_EXT'));
				$thext[$cext] = $cextval;
				$this->setSetting('MY_EXPLORER_HIDDEN_EXT', serialize($thext) );
				return true;
			break;
			case 'delfiles':
			$rp = realpath('./');
				$files = explode(' ', Filter::post('files'));NP::debug('file', $files);
				foreach ($files as $file){NP::debug('rp' , $rp . substr($file,1) );
					unlink(realpath($rp . substr($file,1)));
				}
				unlink(WT_DATA_DIR . $this->getSetting('MY_EXPLORER_CACHE_DIR') . '/' . $this->getSetting('MY_EXPLORER_TAB_FILES'));
				return true;
			break;
			default:
				http_response_code(404);
			break;
			
		}
}

		
		/*
		*	param $main_file path
		*	param $name  target name
		*/
	private  function resizedImage($main_file, $name){
		$IMAGE_WIDTH = 200;
		//image
		$file = WT_DATA_DIR . $this->getName() . '_cache/' . $name;

		try {
				$imgsize = getimagesize($main_file);
				// Image small enough to be its own thumbnail?
				if ($imgsize[0] <= $IMAGE_WIDTH && $imgsize[1] <= $IMAGE_WIDTH) {NP::debug('copy', $file);
					try {
						copy($main_file, $file);
						Log::addMediaLog('resized image created for ' . $main_file . ' (copy of main image)');
					} catch (\ErrorException $ex) {
						Log::addMediaLog('Image resized  could not be created for ' . $main_file . ' (copy of main image)');
					}
				} else {
					if (FunctionsMedia::hasMemoryForImage($main_file)) {
						try {
							
							switch ($imgsize['mime']) {
							case 'image/png':NP::debug('mime: ', $imgsize['mime']);
								$main_image = imagecreatefrompng($main_file);
								break;
							case 'image/gif':
								$main_image = imagecreatefromgif($main_file);
								break;
							case 'image/jpeg':
								$main_image = imagecreatefromjpeg($main_file);
								break;
							default:
								return $file; // Nothing else we can do :-(
							}
							
							if ($main_image) {
								// How big should the image resized be?
								$width       = $IMAGE_WIDTH;
								$height = $width;
								
								$imgAspectRatio = $imgsize[0] / $imgsize[1];
								
						if ($imgAspectRatio > 1){
							$height = round($IMAGE_WIDTH / $imgAspectRatio);
						} elseif ($imgAspectRatio < 1) {	
							$width = round($IMAGE_WIDTH * $imgAspectRatio);
						} 
						
								$thumb_image = imagecreatetruecolor($width, $height);
									
								// Create a transparent background, instead of the default black one
								imagesavealpha($thumb_image, true);
								imagefill($thumb_image, 0, 0, imagecolorallocatealpha($thumb_image, 0, 0, 0, 127));
								// Shrink the image
								imagecopyresampled($thumb_image, $main_image, 0, 0, 0, 0, $width, $height, $imgsize[0], $imgsize[1]);
								
								switch ($imgsize['mime']) {
								case 'image/png':
									imagepng($thumb_image, $file);
									break;
								case 'image/gif':
									imagegif($thumb_image, $file);
									break;
								case 'image/jpeg':
									imagejpeg($thumb_image, $file);
									break;
								}
						//		imagedestroy($main_image);
						//		imagedestroy($thumb_image);
								Log::addMediaLog('Image resized created for ' . $main_file);
						return $file;
							}
						} catch (\ErrorException $ex) {
							Log::addMediaLog('Failed to create thumbnail for ' . $main_file . ' (' . $ex . ')');
						}
					} else {
						Log::addMediaLog('Not enough memory to create thumbnail for ' . $main_file);
					}
				}
			} catch (\ErrorException $ex) {
				// Not an image, or not a valid image?
			}
	return true;
	}

//----------------------------------------------------
	
		/**
	 * What is the mime-type of this object?
	 * For simplicity and efficiency, use the extension, rather than the contents.
	 *
	 * @return string
	 */
	public static function getMimeType($file) {
		// Themes contain icon definitions for some/all of these mime-types
		switch (strtolower(self::getExtFile($file))) {
		case 'bmp':
			return 'image/bmp';
		case 'doc':
			return 'application/msword';
		case 'docx':
			return 'application/msword';
		case 'ged':
			return 'text/x-gedcom';
		case 'gif':
			return 'image/gif';
		case 'htm':
			return 'text/html';
		case 'html':
			return 'text/html';
		case 'jpeg':
			return 'image/jpeg';
		case 'jpg':
			return 'image/jpeg';
		case 'mov':
			return 'video/quicktime';
		case 'mp3':
			return 'audio/mpeg';
		case 'ogv':
			return 'video/ogg';
		case 'pdf':
			return 'application/pdf';
		case 'png':
			return 'image/png';
		case 'rar':
			return 'application/x-rar-compressed';
		case 'swf':
			return 'application/x-shockwave-flash';
		case 'svg':
			return 'image/svg';
		case 'tif':
			return 'image/tiff';
		case 'tiff':
			return 'image/tiff';
		case 'xls':
			return 'application/vnd-ms-excel';
		case 'xlsx':
			return 'application/vnd-ms-excel';
		case 'wmv':
			return 'video/x-ms-wmv';
		case 'zip':
			return 'application/zip';
		default:
			return 'application/octet-stream';
		}
	}

			/*
	*
	*/
	public static function getExtFile($file){
		$ext = (explode('.', $file)); 
		return  end($ext);
	}




	
}
return new myExplorerModule;